//
//  ConnectionViewModel.swift
//  RemoteAccess
//
//  Created by Paweł Leśniewski on 18/11/2024.
//
import Foundation
import LiveKit
import SwiftUI

extension ConnectionView {
    class ViewModel: ObservableObject {
        
        var room: Room? = nil
        
        func initializeNewSession(url: String, token: String) {
            
            print("Starting new session")
            
            let connectOptions = ConnectOptions(
                autoSubscribe: true
            )
            
            let roomOptions = RoomOptions(
                defaultScreenShareCaptureOptions: ScreenShareCaptureOptions(
                    useBroadcastExtension: true
                ),
                defaultVideoPublishOptions: VideoPublishOptions(
                    simulcast: true
                ),
                adaptiveStream: true, // adjusts quality of video tracks based on the size of larges subscribed one
                dynacast: true// pauses video layers if noone subscribe
            )
            
            self.room = Room(delegate: self,
                             connectOptions: connectOptions,
                             roomOptions: roomOptions)
            
            
            Task.do {
                try await self.room!.connect(url: url, token: token)
            } catch: { error in
                print("Something went wrong during connecting to room")
            }
        }
    }
}

/// MARK: Room delegate - LiveKit
extension ConnectionView.ViewModel: RoomDelegate {
    
    func roomDidConnect(_ room: Room) {
        Task.do {
            try await room.localParticipant.setScreenShare(enabled: true)

        } catch: { error in
            print("Something went wrong during triggering broadcast picker [\(error.localizedDescription)]")
        }
    }
    
}

extension Task
where Success == Void,
      Failure == Never {

    typealias ThrowingOperation = () async throws -> Void
    typealias ErrorHandler = (Error) async -> Void

    /// Creates a throwing task with familiar do-catch-like syntax
    @discardableResult
    static func `do`(
        priority: TaskPriority? = nil,
        throwingOperation: @escaping ThrowingOperation,
        `catch` errorHandler: @escaping ErrorHandler
    ) -> Self {
        
        return Task(priority: priority) {
            do { try await throwingOperation() }
            catch { await errorHandler(error) }
        }
    }
}
